@@ -24,6 +24,7 @@
  */
 
 #include "includes.h"
+#include "system/filesys.h"
 
 /* Do this on our own in TRANSFER_BUF_SIZE chunks.
  * It's safe to make direct syscalls to lseek/write here
@@ -43,7 +44,7 @@
 
 static ssize_t default_sys_recvfile(int fromfd,
 			int tofd,
-			SMB_OFF_T offset,
+			off_t offset,
 			size_t count)
 {
 	int saved_errno = 0;
@@ -61,8 +62,8 @@ static ssize_t default_sys_recvfile(int fromfd,
 		return 0;
 	}
 
-	if (tofd != -1 && offset != (SMB_OFF_T)-1) {
-		if (sys_lseek(tofd, offset, SEEK_SET) == -1) {
+	if (tofd != -1 && offset != (off_t)-1) {
+		if (lseek(tofd, offset, SEEK_SET) == -1) {
 			if (errno != ESPIPE) {
 				return -1;
 			}
@@ -139,7 +140,7 @@ static ssize_t default_sys_recvfile(int fromfd,
 
 ssize_t sys_recvfile(int fromfd,
 			int tofd,
-			SMB_OFF_T offset,
+			off_t offset,
 			size_t count)
 {
 	static int pipefd[2] = { -1, -1 };
@@ -229,7 +230,7 @@ ssize_t sys_recvfile(int fromfd,
 
 ssize_t sys_recvfile(int fromfd,
 			int tofd,
-			SMB_OFF_T offset,
+			off_t offset,
 			size_t count)
 {
 	return default_sys_recvfile(fromfd, tofd, offset, count);
@@ -239,6 +240,7 @@ ssize_t sys_recvfile(int fromfd,
 /*****************************************************************
  Throw away "count" bytes from the client socket.
  Returns count or -1 on error.
+ Must only operate on a blocking socket.
 *****************************************************************/
 
 ssize_t drain_socket(int sockfd, size_t count)
@@ -246,6 +248,7 @@ ssize_t drain_socket(int sockfd, size_t count)
 	size_t total = 0;
 	size_t bufsize = MIN(TRANSFER_BUF_SIZE,count);
 	char *buffer = NULL;
+	int old_flags = 0;
 
 	if (count == 0) {
 		return 0;
@@ -256,6 +259,12 @@ ssize_t drain_socket(int sockfd, size_t count)
 		return -1;
 	}
 
+	old_flags = fcntl(sockfd, F_GETFL, 0);
+	if (set_blocking(sockfd, true) == -1) {
+		free(buffer);
+		return -1;
+	}
+
 	while (total < count) {
 		ssize_t read_ret;
 		size_t toread = MIN(bufsize,count - total);
@@ -264,12 +273,17 @@ ssize_t drain_socket(int sockfd, size_t count)
 		read_ret = sys_read(sockfd, buffer, toread);
 		if (read_ret <= 0) {
 			/* EOF or socket error. */
-			free(buffer);
-			return -1;
+			count = (size_t)-1;
+			goto out;
 		}
 		total += read_ret;
 	}
 
+  out:
+
 	free(buffer);
+	if (fcntl(sockfd, F_SETFL, old_flags) == -1) {
+		return -1;
+	}
 	return count;
 }
